package de.intevation.flys.importer;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

import java.math.BigDecimal;

import org.hibernate.SessionFactory;
import org.hibernate.Session;
import org.hibernate.Query;
import org.hibernate.FlushMode;

import de.intevation.flys.backend.SessionFactoryProvider;

import de.intevation.flys.model.WstColumnValue;
import de.intevation.flys.model.WstColumn;
import de.intevation.flys.model.DischargeTableValue;
import de.intevation.flys.model.DischargeTable;
import de.intevation.flys.model.Range;
import de.intevation.flys.model.River;

import org.apache.log4j.Logger;

public class ImporterSession
{
    private static Logger log = Logger.getLogger(ImporterSession.class);

    private static final ThreadLocal<ImporterSession> SESSION =
        new ThreadLocal<ImporterSession>() {
            @Override
            protected ImporterSession initialValue() {
                return new ImporterSession();
            }
        };

    protected Session databaseSession;

    protected Map<IdValueKey, WstColumnValue> wstColumnValues;

    protected Map<IdValueKey, DischargeTableValue> dischargeTableValues;

    protected Map<IdValueKey, Range> ranges;

    public static ImporterSession getInstance() {
        return SESSION.get();
    }

    public ImporterSession() {
        SessionFactory sessionFactory =
            SessionFactoryProvider.createSessionFactory();
        databaseSession = sessionFactory.openSession();
        databaseSession.setFlushMode(FlushMode.MANUAL);
    }

    public Session getDatabaseSession() {
        return databaseSession;
    }

    public WstColumnValue getWstColumnValue(
        WstColumn  column,
        BigDecimal position,
        BigDecimal w
    ) {
        if (wstColumnValues == null) {
            loadWstColumnValues();
        }

        IdValueKey key = new IdValueKey(column.getId(), position, w);

        WstColumnValue wcv = wstColumnValues.get(key);

        if (wcv != null) {
            return wcv;
        }

        wcv = new WstColumnValue(column, position, w);

        databaseSession.save(wcv);

        wstColumnValues.put(key, wcv);

        return wcv;
    }

    protected void loadWstColumnValues() {
        log.info("load wst column values");
        wstColumnValues = new HashMap<IdValueKey, WstColumnValue>();

        Query query = databaseSession.createQuery("from WstColumnValue");

        for (Iterator iter = query.iterate(); iter.hasNext();) {
            WstColumnValue wcv = (WstColumnValue)iter.next();
            wstColumnValues.put(new IdValueKey(wcv), wcv);
        }
        log.info(wstColumnValues.size() + " values loaded");
    }

    public DischargeTableValue getDischargeTableValue(
        DischargeTable table,
        BigDecimal     q,
        BigDecimal     w
    ) {
        if (dischargeTableValues == null) {
            loadDischargeTableValues();
        }

        IdValueKey key = new IdValueKey(table.getId(), q, w);

        DischargeTableValue dtv = dischargeTableValues.get(key);

        if (dtv != null) {
            return dtv;
        }

        dtv = new DischargeTableValue(table, q, w);

        databaseSession.save(dtv);

        dischargeTableValues.put(key, dtv);

        return dtv;
    }

    protected void loadDischargeTableValues() {
        log.info("load discharge table values");

        dischargeTableValues = new HashMap<IdValueKey, DischargeTableValue>();

        Query query = databaseSession.createQuery("from DischargeTableValue");

        for (Iterator iter = query.iterate(); iter.hasNext();) {
            DischargeTableValue dtv = (DischargeTableValue)iter.next();
            dischargeTableValues.put(new IdValueKey(dtv), dtv);
        }
        log.info(dischargeTableValues.size() + " values loaded");
    }

    public Range getRange(River river, BigDecimal a, BigDecimal b) {

        if (ranges == null) {
            loadRanges();
        }

        IdValueKey key = new IdValueKey(river.getId(), a, b);

        Range range = ranges.get(key);

        if (ranges != null) {
            return range;
        }

        range = new Range(a, b, river);

        databaseSession.save(range);

        ranges.put(key, range);

        return range;
    }

    protected void loadRanges() {
        log.info("load ranges");
        ranges = new HashMap<IdValueKey, Range>();

        Query query = databaseSession.createQuery("from Range");

        for (Iterator iter = query.iterate(); iter.hasNext();) {
            Range range = (Range)iter.next();
            ranges.put(new IdValueKey(range), range);
        }
        log.info(ranges.size() + " values loaded");
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
